@@ -8,7 +8,6 @@
 #include "miscadmin.h"
 #include "string.h"
 #include "lib/stringinfo.h"
-#include "orafunc.h"
 #include "catalog/pg_type.h"
 #include "utils/builtins.h"
 #include "utils/date.h"
@@ -16,9 +15,9 @@
 
 #include "shmmc.h"
 #include "pipe.h"
+#include "orafunc.h"
 
 /*
- * First test version 0.0.9
  * @ Pavel Stehule 2006
  */
 
@@ -94,7 +93,7 @@ PG_FUNCTION_INFO_V1(dbms_pipe_pack_message_bigint);
 
 typedef enum {
 	IT_NO_MORE_ITEMS = 0,
-	IT_NUMBER = 9, 
+	IT_NUMBER = 9,
 	IT_VARCHAR = 11,
 	IT_DATE = 12,
 	IT_TIMESTAMPTZ = 13,
@@ -102,7 +101,6 @@ typedef enum {
 	IT_RECORD = 24
 } message_data_type;
 
-
 typedef struct _queue_item {
 	void *ptr;
 	struct _queue_item *next_item;
@@ -121,21 +119,28 @@ typedef struct {
 } pipe;
 
 typedef struct {
-	size_t size;
-	int items_count;
-	char data;
-} message_buffer;
-
-typedef struct {
-	size_t size;
+	int32 size;
 	message_data_type type;
 	Oid tupType;
-	char data;
 } message_data_item;
 
+typedef struct {
+	int32 size;
+	int32 items_count;
+	message_data_item *next;
+} message_buffer;
+
+#define message_buffer_size		(MAXALIGN(sizeof(message_buffer)))
+#define message_buffer_get_content(buf)	((message_data_item *) (((char*)buf)+message_buffer_size))
+
+
+#define message_data_item_size	(MAXALIGN(sizeof(message_data_item)))
+#define message_data_get_content(msg) (((char *)msg) + message_data_item_size)
+#define message_data_item_next(msg) \
+	((message_data_item *) (message_data_get_content(msg) + MAXALIGN(msg->size)))
+
 typedef struct PipesFctx {
 	int pipe_nth;
-	char **values;
 } PipesFctx;
 
 typedef struct
@@ -146,16 +151,14 @@ typedef struct
 	alert_lock *locks;
 	size_t size;
 	unsigned int sid;
-	char data[1];  /* VARIABLE LENGTH ARRAY */
+	vardata data[1]; /* flexible array member */
 } sh_memory;
 
+#define sh_memory_size			(offsetof(sh_memory, data))
 
 message_buffer *output_buffer = NULL;
 message_buffer *input_buffer = NULL;
 
-message_data_item *writer = NULL;
-message_data_item *reader = NULL;
-
 pipe* pipes = NULL;
 LWLockId shmem_lock = NOT_INITIALIZED;
 unsigned int sid;                                 /* session id */
@@ -168,73 +171,65 @@ extern alert_lock  *locks;
  * write on writer size bytes from ptr
  */
 
-static void 
-pack_field(message_buffer *message, message_data_item **writer,
-		   message_data_type type, int size, void *ptr, Oid *tupType)
+static void
+pack_field(message_buffer *buffer, message_data_type type,
+			int32 size, void *ptr, Oid tupType)
 {
-	int l;
-	message_data_item *_wr = *writer;
+	int len;
+	message_data_item *message;
 
-	l = size + sizeof(message_data_item);
-	if (message->size + l > LOCALMSGSZ-sizeof(message_buffer))
-    		ereport(ERROR,             
-                            (errcode(ERRCODE_OUT_OF_MEMORY),
-                             errmsg("out of memory"),
-                             errdetail("Packed message is biger than local buffer."),
-			     errhint("Increase LOCALMSGSZ in 'pipe.h' and recompile library.")));                     
+	len = MAXALIGN(size) + message_data_item_size;
+	if (MAXALIGN(buffer->size) + len > LOCALMSGSZ - message_buffer_size)
+		ereport(ERROR,
+				(errcode(ERRCODE_OUT_OF_MEMORY),
+				 errmsg("out of memory"),
+				 errdetail("Packed message is biger than local buffer."),
+				 errhint("Increase LOCALMSGSZ in 'pipe.h' and recompile library.")));
 
-	if (_wr == NULL)
-		_wr = (message_data_item*)&message->data;
+	if (buffer->next == NULL)
+		buffer->next =  message_buffer_get_content(buffer);
 
-	_wr->size = l;
+	message = buffer->next;
 
-	_wr->type = type;
-	if (tupType != NULL)
-	    _wr->tupType = *tupType;
-	
-	memcpy(&_wr->data, ptr, size);
+	message->size = size;
+	message->type = type;
+	message->tupType = tupType;
 
-	message->size += l;
-	message->items_count +=1;
+	/* padding bytes have to be zeroed - buffer creator is responsible to clear memory */
 
-	_wr = (message_data_item*)((char*)_wr + l);
-	*writer = _wr;
+	memcpy(message_data_get_content(message), ptr, size);
+
+	buffer->size += len;
+	buffer->items_count++;
+	buffer->next = message_data_item_next(message);
 }
 
 
 static void*
-unpack_field(message_buffer *message, message_data_item **reader, 
-			 message_data_type *type, size_t *size, Oid *tupType)
+unpack_field(message_buffer *buffer, message_data_type *type,
+				int32 *size, Oid *tupType)
 {
 	void *ptr;
-	message_data_item *_rd = *reader;
+	message_data_item *message;
 
-	if (_rd == NULL)
-	{
-		_rd = (message_data_item*)&message->data;
-	}
-	if ((message->items_count)--)
-	{
-		*size = _rd->size - sizeof(message_data_item);
-		*type = _rd->type;
-		
-		if (tupType)
-			*tupType = _rd->tupType;
+	Assert(buffer != NULL);
+	Assert(buffer->items_count > 0);
+	Assert(buffer->next != NULL);
 
-		ptr  = (void*)&_rd->data;
+	message = buffer->next;
+	*size = message->size;
+	*type = message->type;
+	*tupType = message->tupType;
+	ptr = message_data_get_content(message);
 
-		_rd = (message_data_item*)((char*)_rd + _rd->size);
-		*reader = message->items_count > 0 ? _rd : NULL;
+	buffer->next = --buffer->items_count > 0 ? message_data_item_next(message) : NULL;
 
-		return ptr;
-	}
-
-	return NULL;
+	return ptr;
 }
 
 
 /*
- * Add ptr to queue. If pipe doesn't exist, regigister new pipe
+ * Add ptr to queue. If pipe doesn't exist, register new pipe
  */
 
 bool
@@ -250,26 +245,26 @@ ora_lock_shmem(size_t size, int max_pipes, int max_events, int max_locks, bool r
 		sh_mem = ShmemInitStruct("dbms_pipe", size, &found);
 		uid = GetUserId();
 		if (sh_mem == NULL)
-    			ereport(ERROR,             
-                        	(errcode(ERRCODE_OUT_OF_MEMORY),
-                                 errmsg("out of memory"),
-                                 errdetail("Failed while allocation block %d bytes in shared memory.", size))); 
+			ereport(ERROR,
+					(errcode(ERRCODE_OUT_OF_MEMORY),
+					 errmsg("out of memory"),
+					 errdetail("Failed while allocation block %lu bytes in shared memory.", (unsigned long) size)));
 
 		if (!found)
 		{
 			shmem_lock = sh_mem->shmem_lock = LWLockAssign();
 			LWLockAcquire(sh_mem->shmem_lock, LW_EXCLUSIVE);
-			sh_mem->size = size - sizeof(sh_memory);
-			ora_sinit(&sh_mem->data, size, true);
+			sh_mem->size = size - sh_memory_size;
+			ora_sinit(sh_mem->data, size, true);
 			pipes = sh_mem->pipes = ora_salloc(max_pipes*sizeof(pipe));
-			sid = sh_mem->sid = 1;			
-			for(i = 0; i < max_pipes; i++)
+			sid = sh_mem->sid = 1;
+			for (i = 0; i < max_pipes; i++)
 				pipes[i].is_valid = false;
 
 			events = sh_mem->events = ora_salloc(max_events*sizeof(alert_event));
 			locks = sh_mem->locks = ora_salloc(max_locks*sizeof(alert_lock));
 
-			for(i = 0; i < max_events; i++)
+			for (i = 0; i < max_events; i++)
 			{
 				events[i].event_name = NULL;
 				events[i].max_receivers = 0;
@@ -281,14 +276,14 @@ ora_lock_shmem(size_t size, int max_pipes, int max_events, int max_locks, bool r
 				locks[i].sid = -1;
 				locks[i].echo = NULL;
 			}
-												 
+
 		}
 		else if (sh_mem->shmem_lock != 0)
 		{
 			pipes = sh_mem->pipes;
 			shmem_lock = sh_mem->shmem_lock;
 			LWLockAcquire(sh_mem->shmem_lock, LW_EXCLUSIVE);
-			ora_sinit(&sh_mem->data, sh_mem->size, reset);
+			ora_sinit(sh_mem->data, sh_mem->size, reset);
 			sid = ++(sh_mem->sid);
 			events = sh_mem->events;
 			locks = sh_mem->locks;
@@ -303,7 +298,7 @@ ora_lock_shmem(size_t size, int max_pipes, int max_events, int max_locks, bool r
 		elog(ERROR, "Can't purge memory");
 */
 
-	return pipes != NULL;	
+	return pipes != NULL;
 }
 
 
@@ -320,7 +315,7 @@ find_pipe(text* pipe_name, bool* created, bool only_check)
 	*created = false;
 	for (i = 0; i < MAX_PIPES; i++)
 	{
-		if (pipes[i].is_valid && 
+		if (pipes[i].is_valid &&
 			strncmp((char*)VARDATA(pipe_name), pipes[i].pipe_name, VARSIZE(pipe_name) - VARHDRSZ) == 0
 			&& (strlen(pipes[i].pipe_name) == (VARSIZE(pipe_name) - VARHDRSZ)))
 		{
@@ -329,10 +324,10 @@ find_pipe(text* pipe_name, bool* created, bool only_check)
 			if (pipes[i].creator != NULL && pipes[i].uid != uid)
 			{
 				LWLockRelease(shmem_lock);
-    				ereport(ERROR,             
-                        		(errcode(ERRCODE_INSUFFICIENT_PRIVILEGE),
-                                	 errmsg("insufficient privilege"),
-                                	 errdetail("Insufficient privilege to access pipe"))); 
+				ereport(ERROR,
+						(errcode(ERRCODE_INSUFFICIENT_PRIVILEGE),
+						 errmsg("insufficient privilege"),
+						 errdetail("Insufficient privilege to access pipe")));
 			}
 
 			return &pipes[i];
@@ -360,7 +355,7 @@ find_pipe(text* pipe_name, bool* created, bool only_check)
 			break;
 		}
 
-	return result;	
+	return result;
 }
 
 
@@ -384,7 +379,7 @@ new_last(pipe *p, void *ptr)
 	q = p->items;
 	while (q->next_item != NULL)
 		q = q->next_item;
-	
+
 
 	if (NULL == (aux_q = ora_salloc(sizeof(queue_item))))
 		return false;
@@ -475,17 +470,17 @@ add_to_pipe(text *pipe_name, message_buffer *ptr, int limit, bool limit_is_valid
 
 	if (!ora_lock_shmem(SHMEMMSGSZ, MAX_PIPES, MAX_EVENTS, MAX_LOCKS,false))
 		return false;
-		
+
 	for (;;)
 	{
 		if (NULL != (p = find_pipe(pipe_name, &created, false)))
 		{
 			if (created)
 				p->registered = ptr == NULL;
-		
+
 			if (limit_is_valid && (created || (p->limit < limit)))
 				p->limit = limit;
-		
+
 			if (ptr != NULL)
 			{
 				if (NULL != (sh_ptr = ora_salloc(ptr->size)))
@@ -505,7 +500,7 @@ add_to_pipe(text *pipe_name, message_buffer *ptr, int limit, bool limit_is_valid
 					ora_sfree(p->pipe_name);
 					p->is_valid = false;
 					result = false;
-				}	
+				}
 			}
 			else
 				result = true;
@@ -551,33 +546,45 @@ remove_pipe(text *pipe_name, bool purge)
 Datum
 dbms_pipe_next_item_type (PG_FUNCTION_ARGS)
 {
-	
-	PG_RETURN_INT32(reader != NULL ? reader->type : IT_NO_MORE_ITEMS);
+	PG_RETURN_INT32(input_buffer != NULL ? input_buffer->next->type : IT_NO_MORE_ITEMS);
+}
+
+
+static void
+init_buffer(message_buffer *buffer, int32 size)
+{
+	memset(buffer, 0, size);
+	buffer->size = message_buffer_size;
+	buffer->items_count = 0;
+	buffer->next = message_buffer_get_content(buffer);
 }
 
 static message_buffer*
-check_buffer(message_buffer *buf, size_t size, message_data_item **writer)
+check_buffer(message_buffer *buffer, int32 size)
 {
-	if (buf == NULL)
+	if (buffer == NULL)
 	{
-		buf = (message_buffer*) MemoryContextAlloc(TopMemoryContext, size);
-		buf->size = sizeof(message_buffer);
-		buf->items_count = 0;
-		*writer = (message_data_item*) &buf->data;
-	}	
+		buffer = (message_buffer*) MemoryContextAlloc(TopMemoryContext, size);
+		if (buffer == NULL)
+			ereport(ERROR,
+					(errcode(ERRCODE_OUT_OF_MEMORY),
+					 errmsg("out of memory"),
+					 errdetail("Failed while allocation block %d bytes in memory.", size)));
 
-	return buf;
+		init_buffer(buffer, size);
+	}
+
+	return buffer;
 }
 
 Datum
 dbms_pipe_pack_message_text(PG_FUNCTION_ARGS)
 {
-	text *str = PG_GETARG_TEXT_P(0);
+	text *str = PG_GETARG_TEXT_PP(0);
 
-	output_buffer = check_buffer(output_buffer, LOCALMSGSZ, &writer);	
-	pack_field(output_buffer, &writer, IT_VARCHAR, 
-			   VARSIZE(str) - VARHDRSZ, VARDATA(str),
-			   NULL); 
+	output_buffer = check_buffer(output_buffer, LOCALMSGSZ);
+	pack_field(output_buffer, IT_VARCHAR,
+		VARSIZE_ANY_EXHDR(str), VARDATA_ANY(str), InvalidOid);
 
 	PG_RETURN_VOID();
 }
@@ -588,10 +595,9 @@ dbms_pipe_pack_message_date(PG_FUNCTION_ARGS)
 {
 	DateADT dt = PG_GETARG_DATEADT(0);
 
-	output_buffer = check_buffer(output_buffer, LOCALMSGSZ, &writer);	
-	pack_field(output_buffer, &writer, IT_DATE,
-			   sizeof(dt), &dt,
-			   NULL); 
+	output_buffer = check_buffer(output_buffer, LOCALMSGSZ);
+	pack_field(output_buffer, IT_DATE,
+			   sizeof(dt), &dt, InvalidOid);
 
 	PG_RETURN_VOID();
 }
@@ -602,13 +608,11 @@ dbms_pipe_pack_message_timestamp(PG_FUNCTION_ARGS)
 {
 	TimestampTz dt = PG_GETARG_TIMESTAMPTZ(0);
 
-	output_buffer = check_buffer(output_buffer, LOCALMSGSZ, &writer);	
-	pack_field(output_buffer, &writer, IT_TIMESTAMPTZ,
-			   sizeof(dt), &dt,
-			   NULL); 
+	output_buffer = check_buffer(output_buffer, LOCALMSGSZ);
+	pack_field(output_buffer, IT_TIMESTAMPTZ,
+			   sizeof(dt), &dt, InvalidOid);
 
 	PG_RETURN_VOID();
-
 }
 
 
@@ -617,13 +621,11 @@ dbms_pipe_pack_message_number(PG_FUNCTION_ARGS)
 {
 	Numeric num = PG_GETARG_NUMERIC(0);
 
-	output_buffer = check_buffer(output_buffer, LOCALMSGSZ, &writer);	
-	pack_field(output_buffer, &writer, IT_NUMBER,
-			   VARSIZE(num) - VARHDRSZ, VARDATA(num),
-			   NULL); 
+	output_buffer = check_buffer(output_buffer, LOCALMSGSZ);
+	pack_field(output_buffer, IT_NUMBER,
+			   VARSIZE(num) - VARHDRSZ, VARDATA(num), InvalidOid);
 
 	PG_RETURN_VOID();
-
 }
 
 
@@ -632,13 +634,11 @@ dbms_pipe_pack_message_bytea(PG_FUNCTION_ARGS)
 {
 	bytea *data = PG_GETARG_BYTEA_P(0);
 
-	output_buffer = check_buffer(output_buffer, LOCALMSGSZ, &writer);	
-	pack_field(output_buffer, &writer, IT_BYTEA,
-			   VARSIZE(data) - VARHDRSZ, VARDATA(data),
-			   NULL); 
+	output_buffer = check_buffer(output_buffer, LOCALMSGSZ);
+	pack_field(output_buffer, IT_BYTEA,
+		VARSIZE_ANY_EXHDR(data), VARDATA_ANY(data), InvalidOid);
 
 	PG_RETURN_VOID();
-
 }
 
 
@@ -649,233 +649,159 @@ dbms_pipe_pack_message_bytea(PG_FUNCTION_ARGS)
 Datum
 dbms_pipe_pack_message_record(PG_FUNCTION_ARGS)
 {
-#ifndef PG_VERSION_74_COMPAT
 	HeapTupleHeader rec = PG_GETARG_HEAPTUPLEHEADER(0);
 	Oid tupType;
 	bytea *data;
-	FunctionCallInfoData locfcinfo;
+	FunctionCallInfoData info;
 
 	tupType = HeapTupleHeaderGetTypeId(rec);
-	
+
 	/*
 	 * Normally one would call record_send() using DirectFunctionCall3,
 	 * but that does not work since record_send wants to cache some data
 	 * using fcinfo->flinfo->fn_extra.  So we need to pass it our own
 	 * flinfo parameter.
 	 */
-	InitFunctionCallInfoData(locfcinfo, fcinfo->flinfo, 3, NULL, NULL);
+	InitFunctionCallInfoData(info, fcinfo->flinfo, 3, NULL, NULL);
 
-	locfcinfo.arg[0] = PointerGetDatum(rec);
-	locfcinfo.arg[1] = ObjectIdGetDatum(tupType);
-	locfcinfo.arg[2] = Int32GetDatum(-1);
-	locfcinfo.argnull[0] = false;
-	locfcinfo.argnull[1] = false;
-	locfcinfo.argnull[2] = false;
-			
-	data = (bytea*) DatumGetPointer(record_send(&locfcinfo));
+	info.arg[0] = PointerGetDatum(rec);
+	info.arg[1] = ObjectIdGetDatum(tupType);
+	info.arg[2] = Int32GetDatum(-1);
+	info.argnull[0] = false;
+	info.argnull[1] = false;
+	info.argnull[2] = false;
 
-	output_buffer = check_buffer(output_buffer, LOCALMSGSZ, &writer);	
-	pack_field(output_buffer, &writer, IT_RECORD,
-			   VARSIZE(data), VARDATA(data),
-			   &tupType); 
-#else
-	ereport(ERROR,
-                (errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-		 errmsg("feature not supported"),
-		 errdetail("PostgreSQL 7.4 doesn't allow send or receive record")));
-#endif
+	data = (bytea*) DatumGetPointer(record_send(&info));
+
+	output_buffer = check_buffer(output_buffer, LOCALMSGSZ);
+	pack_field(output_buffer, IT_RECORD,
+			   VARSIZE(data), VARDATA(data), tupType);
 
 	PG_RETURN_VOID();
 }
 
 
-static int64
-dbms_pipe_unpack_message(message_data_type dtype, bool *is_null, Oid *tupType)
+static Datum
+dbms_pipe_unpack_message(PG_FUNCTION_ARGS, message_data_type dtype)
 {
+	Oid		tupType;
 	void *ptr;
 	message_data_type type;
-	size_t size;
-	int64 result = 0;
+	int32 size;
+	Datum result;
 	message_data_type next_type;
 
-	*is_null = true;
-	if (input_buffer == NULL)
-		return result;
-
-	next_type =  reader != NULL ? reader->type : IT_NO_MORE_ITEMS;
-	if (next_type == IT_NO_MORE_ITEMS)
-		return result;
+	if (input_buffer == NULL ||
+		input_buffer->items_count <= 0 ||
+		input_buffer->next == NULL ||
+		input_buffer->next->type == IT_NO_MORE_ITEMS)
+		PG_RETURN_NULL();
 
+	next_type = input_buffer->next->type;
 	if (next_type != dtype)
-		ereport(ERROR,             
+		ereport(ERROR,
 				(errcode(ERRCODE_DATATYPE_MISMATCH),
 				 errmsg("datatype mismatch"),
-				 errdetail("unpack unexpected type"))); 
-	
-	if (NULL == (ptr = unpack_field(input_buffer, &reader, &type, &size, tupType)))
-		return result;
+				 errdetail("unpack unexpected type: %d", next_type)));
+
+	ptr = unpack_field(input_buffer, &type, &size, &tupType);
+	Assert(ptr != NULL);
 
 	switch (type)
 	{
 		case IT_TIMESTAMPTZ:
-			result = *(int64*)ptr;
+			result = TimestampTzGetDatum(*(TimestampTz*)ptr);
 			break;
-
 		case IT_DATE:
-			result = *(DateADT*)ptr;
+			result = DateADTGetDatum(*(DateADT*)ptr);
 			break;
-
 		case IT_VARCHAR:
 		case IT_NUMBER:
 		case IT_BYTEA:
-		case IT_RECORD:
-			result = (int64)((int32)ora_make_text_fix((char*)ptr, size));
-			break;			
-	
-		default:
-			*is_null = true;
+			result = PointerGetDatum(cstring_to_text_with_len(ptr, size));
 			break;
-	}
+		case IT_RECORD:
+		{
+			FunctionCallInfoData	info;
+			StringInfoData	buf;
+			text		   *data = cstring_to_text_with_len(ptr, size);
 
-	*is_null = false;
+			buf.data = VARDATA(data);
+			buf.len = VARSIZE(data) - VARHDRSZ;
+			buf.maxlen = buf.len;
+			buf.cursor = 0;
+
+			/*
+			 * Normally one would call record_recv() using DirectFunctionCall3,
+			 * but that does not work since record_recv wants to cache some data
+			 * using fcinfo->flinfo->fn_extra.  So we need to pass it our own
+			 * flinfo parameter.
+			 */
+			InitFunctionCallInfoData(info, fcinfo->flinfo, 3, NULL, NULL);
+
+			info.arg[0] = PointerGetDatum(&buf);
+			info.arg[1] = ObjectIdGetDatum(tupType);
+			info.arg[2] = Int32GetDatum(-1);
+			info.argnull[0] = false;
+			info.argnull[1] = false;
+			info.argnull[2] = false;
+
+			result = record_recv(&info);
+			break;
+		}
+		default:
+			elog(ERROR, "unexpected type: %d", type);
+			result = (Datum) 0;	/* keep compiler quiet */
+	}
 
 	if (input_buffer->items_count == 0)
 	{
 		pfree(input_buffer);
 		input_buffer = NULL;
 	}
-	
-	return result;
+
+	PG_RETURN_DATUM(result);
 }
 
 
 Datum
 dbms_pipe_unpack_message_text(PG_FUNCTION_ARGS)
 {
-	bool is_null;
-	text *result;
-	
-	result = (text*)((int32)dbms_pipe_unpack_message(IT_VARCHAR, &is_null,
-													 NULL));
-	if (is_null)
-		PG_RETURN_NULL();
-	else
-		PG_RETURN_TEXT_P(result);
+	return dbms_pipe_unpack_message(fcinfo, IT_VARCHAR);
 }
 
 
 Datum
 dbms_pipe_unpack_message_date(PG_FUNCTION_ARGS)
 {
-	bool is_null;
-	DateADT result;
-	
-	result = (DateADT)dbms_pipe_unpack_message(IT_DATE, &is_null,
-											   NULL);
-	if (is_null)
-		PG_RETURN_NULL();
-	else
-		PG_RETURN_DATEADT(result);
+	return dbms_pipe_unpack_message(fcinfo, IT_DATE);
 }
 
 Datum
 dbms_pipe_unpack_message_timestamp(PG_FUNCTION_ARGS)
 {
-	bool is_null;
-	TimestampTz result;
-
-	int64 value;
-
-	value = dbms_pipe_unpack_message(IT_TIMESTAMPTZ, &is_null, 
-									 NULL);
-	if (is_null)
-		PG_RETURN_NULL();
-	else
-		result = *((TimestampTz*)&value);
-		PG_RETURN_TIMESTAMPTZ(result);
+	return dbms_pipe_unpack_message(fcinfo, IT_TIMESTAMPTZ);
 }
 
 
 Datum
 dbms_pipe_unpack_message_number(PG_FUNCTION_ARGS)
 {
-	bool is_null;
-	Numeric result;
-
-	result = (Numeric)((int32)dbms_pipe_unpack_message(IT_NUMBER, &is_null,
-													   NULL));
-	if (is_null)
-		PG_RETURN_NULL();
-	else
-		PG_RETURN_NUMERIC(result);
+	return dbms_pipe_unpack_message(fcinfo, IT_NUMBER);
 }
 
 
 Datum
 dbms_pipe_unpack_message_bytea(PG_FUNCTION_ARGS)
 {
-	bool is_null;
-	bytea *result;
-
-	result = (bytea*)((int32)dbms_pipe_unpack_message(IT_BYTEA, &is_null,
-													  NULL));
-	if (is_null)
-		PG_RETURN_NULL();
-	else
-		PG_RETURN_BYTEA_P(result);
+	return dbms_pipe_unpack_message(fcinfo, IT_BYTEA);
 }
 
 
 Datum
 dbms_pipe_unpack_message_record(PG_FUNCTION_ARGS)
 {
-#ifndef PG_VERSION_74_COMPAT
-	bool is_null;
-	bytea *data;
-	FunctionCallInfoData locfcinfo;
-	Oid tupType;
-	HeapTupleHeader rec; 
-	StringInfoData buf;
-
-	data = (bytea*)((int32)dbms_pipe_unpack_message(IT_RECORD, &is_null, &tupType));
-	
-	if (!is_null)
-	{												
-		buf.data = VARDATA(data);
-		buf.len = VARSIZE(data) - VARHDRSZ;
-		buf.maxlen = buf.len;
-		buf.cursor = 0;
-	
-		/*
-		 * Normally one would call record_recv() using DirectFunctionCall3,
-		 * but that does not work since record_recv wants to cache some data
-		 * using fcinfo->flinfo->fn_extra.  So we need to pass it our own
-		 * flinfo parameter.
-		 */
-		InitFunctionCallInfoData(locfcinfo, fcinfo->flinfo, 3, NULL, NULL);
-		
-		locfcinfo.arg[0] = PointerGetDatum(&buf);
-		locfcinfo.arg[1] = ObjectIdGetDatum(tupType);
-		locfcinfo.arg[2] = Int32GetDatum(-1);
-		locfcinfo.argnull[0] = false;
-		locfcinfo.argnull[1] = false;
-		locfcinfo.argnull[2] = false;
-		
-		rec =  DatumGetHeapTupleHeader(record_recv(&locfcinfo));
-	
-		/* I have to translate rec to tuple */
-	
-		pfree(data);
-		PG_RETURN_HEAPTUPLEHEADER(rec);
-	}
-#else
-	ereport(ERROR,
-                (errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-		 errmsg("feature not supported"),
-		 errdetail("PostgreSQL 7.4 doesn't allow send or receive record")));
-#endif
-
-	PG_RETURN_NULL();
+	return dbms_pipe_unpack_message(fcinfo, IT_RECORD);
 }
 
 
@@ -896,17 +822,17 @@ pg_usleep(10000L); \
 Datum
 dbms_pipe_receive_message(PG_FUNCTION_ARGS)
 {
-	text *pipe_name = NULL; 
+	text *pipe_name = NULL;
 	int timeout = ONE_YEAR;
 	int cycle = 0;
 	float8 endtime;
 	bool found = false;
 
 	if (PG_ARGISNULL(0))
-    		ereport(ERROR,
-                        (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
-                         errmsg("pipe name is NULL"),
-                         errdetail("Pipename may not be NULL.")));
+		ereport(ERROR,
+				(errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
+				 errmsg("pipe name is NULL"),
+				 errdetail("Pipename may not be NULL.")));
 	else
 		pipe_name = PG_GETARG_TEXT_P(0);
 
@@ -914,15 +840,15 @@ dbms_pipe_receive_message(PG_FUNCTION_ARGS)
 		timeout = PG_GETARG_INT32(1);
 
 	if (input_buffer != NULL)
+	{
 		pfree(input_buffer);
-	
-	input_buffer = NULL;
-	reader = NULL;
-  
-	WATCH_PRE(timeout, endtime, cycle);	
+		input_buffer = NULL;
+	}
+
+	WATCH_PRE(timeout, endtime, cycle);
 	if (NULL != (input_buffer = get_from_pipe(pipe_name, &found)))
 	{
-		reader = (message_data_item*)&input_buffer->data;
+		input_buffer->next = message_buffer_get_content(input_buffer);
 		break;
 	}
 /* found empty message */
@@ -937,7 +863,7 @@ dbms_pipe_receive_message(PG_FUNCTION_ARGS)
 Datum
 dbms_pipe_send_message(PG_FUNCTION_ARGS)
 {
-	text *pipe_name = NULL; 
+	text *pipe_name = NULL;
 	int timeout = ONE_YEAR;
 	int limit = 0;
 	bool valid_limit;
@@ -946,19 +872,14 @@ dbms_pipe_send_message(PG_FUNCTION_ARGS)
 	float8 endtime;
 
 	if (PG_ARGISNULL(0))
-    		ereport(ERROR,
-                        (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
-                         errmsg("pipe name is NULL"),
-                         errdetail("Pipename may not be NULL.")));
+		ereport(ERROR,
+				(errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
+				 errmsg("pipe name is NULL"),
+				 errdetail("Pipename may not be NULL.")));
 	else
 		pipe_name = PG_GETARG_TEXT_P(0);
 
-	if (output_buffer == NULL)
-	{
-		output_buffer = (message_buffer*) MemoryContextAlloc(TopMemoryContext, LOCALMSGSZ);
-		output_buffer->size = sizeof(message_buffer);
-		output_buffer->items_count = 0;
-	}	
+	output_buffer = check_buffer(output_buffer, LOCALMSGSZ);
 
 	if (!PG_ARGISNULL(1))
 		timeout = PG_GETARG_INT32(1);
@@ -971,21 +892,19 @@ dbms_pipe_send_message(PG_FUNCTION_ARGS)
 		valid_limit = true;
 	}
 
-	if (input_buffer != NULL)
+	if (input_buffer != NULL) /* XXX Strange? */
+	{
 		pfree(input_buffer);
+		input_buffer = NULL;
+	}
 
-	input_buffer = NULL;
-	reader = NULL;
-
-	WATCH_PRE(timeout, endtime, cycle);	
-	if (add_to_pipe(pipe_name, output_buffer, 
+	WATCH_PRE(timeout, endtime, cycle);
+	if (add_to_pipe(pipe_name, output_buffer,
 					limit, valid_limit))
 		break;
 	WATCH_POST(timeout, endtime, cycle);
-		
-	output_buffer->items_count = 0;
-	output_buffer->size = sizeof(message_buffer);
-	writer = (message_data_item*)&output_buffer->data;
+
+	init_buffer(output_buffer, LOCALMSGSZ);
 
 	PG_RETURN_INT32(RESULT_DATA);
 }
@@ -1001,13 +920,13 @@ dbms_pipe_unique_session_name (PG_FUNCTION_ARGS)
 	int cycle = 0;
 	int timeout = 10;
 
-	WATCH_PRE(timeout, endtime, cycle);	
+	WATCH_PRE(timeout, endtime, cycle);
 	if (ora_lock_shmem(SHMEMMSGSZ, MAX_PIPES,MAX_EVENTS,MAX_LOCKS,false))
 	{
 		initStringInfo(&strbuf);
 		appendStringInfo(&strbuf,"PG$PIPE$%d$%d",sid, MyProcPid);
-		
-		result = ora_make_text_fix(strbuf.data, strbuf.len);
+
+		result = cstring_to_text_with_len(strbuf.data, strbuf.len);
 		pfree(strbuf.data);
 		LWLockRelease(shmem_lock);
 
@@ -1015,13 +934,14 @@ dbms_pipe_unique_session_name (PG_FUNCTION_ARGS)
 	}
 	WATCH_POST(timeout, endtime, cycle);
 	LOCK_ERROR();
-	
+
 	PG_RETURN_NULL();
 }
 
+#define DB_PIPES_COLS		6
 
 Datum
-dbms_pipe_list_pipes (PG_FUNCTION_ARGS)
+dbms_pipe_list_pipes(PG_FUNCTION_ARGS)
 {
 	FuncCallContext *funcctx;
 	TupleDesc        tupdesc;
@@ -1031,15 +951,16 @@ dbms_pipe_list_pipes (PG_FUNCTION_ARGS)
 
 	float8 endtime;
 	int cycle = 0;
-	int timeout = 10;	
+	int timeout = 10;
 
-	if (SRF_IS_FIRSTCALL ())
+	if (SRF_IS_FIRSTCALL())
 	{
+		int		i;
 		MemoryContext  oldcontext;
 		bool has_lock = false;
 
-		WATCH_PRE(timeout, endtime, cycle);	
-		if (ora_lock_shmem(SHMEMMSGSZ, MAX_PIPES,MAX_EVENTS,MAX_LOCKS,false))
+		WATCH_PRE(timeout, endtime, cycle);
+		if (ora_lock_shmem(SHMEMMSGSZ, MAX_PIPES, MAX_EVENTS, MAX_LOCKS, false))
 		{
 			has_lock = true;
 			break;
@@ -1048,98 +969,82 @@ dbms_pipe_list_pipes (PG_FUNCTION_ARGS)
 		if (!has_lock)
 			LOCK_ERROR();
 
-		funcctx = SRF_FIRSTCALL_INIT ();
-		oldcontext = MemoryContextSwitchTo (funcctx->multi_call_memory_ctx);
+		funcctx = SRF_FIRSTCALL_INIT();
+		oldcontext = MemoryContextSwitchTo(funcctx->multi_call_memory_ctx);
 
-		fctx = (PipesFctx*) palloc (sizeof (PipesFctx));
-		funcctx->user_fctx = (void *)fctx;
-
-		fctx->values = (char **) palloc (4 * sizeof (char *));
-		fctx->values  [0] = (char*) palloc (255 * sizeof (char));
-		fctx->values  [1] = (char*) palloc  (16 * sizeof (char));
-		fctx->values  [2] = (char*) palloc  (16 * sizeof (char));
-		fctx->values  [3] = (char*) palloc  (16 * sizeof (char));
-		fctx->values  [4] = (char*) palloc  (10 * sizeof (char));
-		fctx->values  [5] = (char*) palloc (255 * sizeof (char));
+		fctx = palloc(sizeof(PipesFctx));
+		funcctx->user_fctx = fctx;
 		fctx->pipe_nth = 0;
 
-		tupdesc = CreateTemplateTupleDesc (6 , false);
+		tupdesc = CreateTemplateTupleDesc(DB_PIPES_COLS , false);
+		i = 0;
+		TupleDescInitEntry(tupdesc, ++i, "name",    VARCHAROID, -1, 0);
+		TupleDescInitEntry(tupdesc, ++i, "items",   INT4OID,    -1, 0);
+		TupleDescInitEntry(tupdesc, ++i, "size",    INT4OID,    -1, 0);
+		TupleDescInitEntry(tupdesc, ++i, "limit",   INT4OID,    -1, 0);
+		TupleDescInitEntry(tupdesc, ++i, "private", BOOLOID,    -1, 0);
+		TupleDescInitEntry(tupdesc, ++i, "owner",   VARCHAROID, -1, 0);
+		Assert(i == DB_PIPES_COLS);
 
-#ifndef PG_VERSION_74_COMPAT 
-		TupleDescInitEntry (tupdesc,  1, "Name",    VARCHAROID, -1, 0);
-		TupleDescInitEntry (tupdesc,  2, "Items",   INT4OID   , -1, 0);
-		TupleDescInitEntry (tupdesc,  3, "Size",    INT4OID,    -1, 0);
-		TupleDescInitEntry (tupdesc,  4, "Limit",   INT4OID,    -1, 0);
-		TupleDescInitEntry (tupdesc,  5, "Private", BOOLOID,    -1, 0);
-		TupleDescInitEntry (tupdesc,  6, "Owner",   VARCHAROID, -1, 0);
-#else
-		TupleDescInitEntry (tupdesc,  1, "Name",    VARCHAROID, -1, 0, false);
-		TupleDescInitEntry (tupdesc,  2, "Items",   INT4OID   , -1, 0, false);
-		TupleDescInitEntry (tupdesc,  3, "Size",    INT4OID,    -1, 0, false);
-		TupleDescInitEntry (tupdesc,  4, "Limit",   INT4OID,    -1, 0, false);
-		TupleDescInitEntry (tupdesc,  5, "Private", BOOLOID,    -1, 0, false);
-		TupleDescInitEntry (tupdesc,  6, "Owner",   VARCHAROID, -1, 0, false);
-#endif
-		
-		slot = TupleDescGetSlot (tupdesc); 
-		funcctx -> slot = slot;
-		
-		attinmeta = TupleDescGetAttInMetadata (tupdesc);
-		funcctx -> attinmeta = attinmeta;
-		
-		MemoryContextSwitchTo (oldcontext);
+		slot = TupleDescGetSlot(tupdesc);
+		funcctx->slot = slot;
+
+		attinmeta = TupleDescGetAttInMetadata(tupdesc);
+		funcctx->attinmeta = attinmeta;
+
+		MemoryContextSwitchTo(oldcontext);
 	}
 
-	funcctx = SRF_PERCALL_SETUP ();
-	fctx = (PipesFctx*) funcctx->user_fctx;
+	funcctx = SRF_PERCALL_SETUP();
+	fctx = (PipesFctx *) funcctx->user_fctx;
 
 	while (fctx->pipe_nth < MAX_PIPES)
 	{
 		if (pipes[fctx->pipe_nth].is_valid)
 		{
-			Datum    result;
-			char   **values;
-			HeapTuple tuple;
-			char     *aux_3, *aux_5;
+			Datum		result;
+			HeapTuple	tuple;
+			char	   *values[DB_PIPES_COLS];
+			char		items[16];
+			char		size[16];
+			char		limit[16];
 
-			values = fctx->values;
-			aux_3 = values[3]; aux_5 = values[5];
-			values[3] = NULL; values[5] = NULL;
-
-			snprintf (values[0], 255, "%s", pipes[fctx->pipe_nth].pipe_name);
-			snprintf (values[1],  16, "%d", pipes[fctx->pipe_nth].count);
-			snprintf (values[2],  16, "%d", pipes[fctx->pipe_nth].size);
+			/* name */
+			values[0] = pipes[fctx->pipe_nth].pipe_name;
+			/* items */
+			snprintf(items, lengthof(items), "%d", pipes[fctx->pipe_nth].count);
+			values[1] = items;
+			/* items */
+			snprintf(size, lengthof(size), "%d", pipes[fctx->pipe_nth].size);
+			values[2] = size;
+			/* limit */
 			if (pipes[fctx->pipe_nth].limit != -1)
 			{
-				snprintf (aux_3,  16, "%d", pipes[fctx->pipe_nth].limit);
-				values[3] = aux_3;
+				snprintf(limit, lengthof(limit), "%d", pipes[fctx->pipe_nth].limit);
+				values[3] = limit;
+			}
+			else
+				values[3] = NULL;
+			/* private */
+			values[4] = (pipes[fctx->pipe_nth].creator ? "true" : "false");
+			/* owner */
+			values[5] = pipes[fctx->pipe_nth].creator;
+
+			tuple = BuildTupleFromCStrings(funcctx->attinmeta, values);
+			result = TupleGetDatum(funcctx->slot, tuple);
 
-			}
-			snprintf (values[4], 10, "%s", pipes[fctx->pipe_nth].creator != NULL ? "true" : "false");
-			if (pipes[fctx->pipe_nth].creator != NULL)
-			{
-				snprintf (aux_5,  255, "%s", pipes[fctx->pipe_nth].creator);
-				values[5] = aux_5;
-			}
-				
-			tuple = BuildTupleFromCStrings (funcctx -> attinmeta,
-											fctx -> values);
-			result = TupleGetDatum (funcctx -> slot, tuple);
-			
-			values[3] = aux_3; values[5] = aux_5;
 			fctx->pipe_nth += 1;
-			SRF_RETURN_NEXT (funcctx, result);
+			SRF_RETURN_NEXT(funcctx, result);
 		}
 		fctx->pipe_nth += 1;
-			
 	}
 
-	LWLockRelease(shmem_lock);	
-	SRF_RETURN_DONE (funcctx);
+	LWLockRelease(shmem_lock);
+	SRF_RETURN_DONE(funcctx);
 }
 
 /*
- * secondary functions 
+ * secondary functions
  */
 
 /*
@@ -1147,7 +1052,7 @@ dbms_pipe_list_pipes (PG_FUNCTION_ARGS)
  *   dbms_pipe.create_pipe(pipe_name varchar, limit := -1 int, private := false bool);
  */
 
-Datum 
+Datum
 dbms_pipe_create_pipe (PG_FUNCTION_ARGS)
 {
 	text *pipe_name = NULL;
@@ -1160,10 +1065,10 @@ dbms_pipe_create_pipe (PG_FUNCTION_ARGS)
 	int timeout = 10;
 
 	if (PG_ARGISNULL(0))
-    		ereport(ERROR,
-                        (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
-                         errmsg("pipe name is NULL"),
-                         errdetail("Pipename may not be NULL.")));
+		ereport(ERROR,
+				(errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
+				 errmsg("pipe name is NULL"),
+				 errdetail("Pipename may not be NULL.")));
 	else
 		pipe_name = PG_GETARG_TEXT_P(0);
 
@@ -1175,7 +1080,7 @@ dbms_pipe_create_pipe (PG_FUNCTION_ARGS)
 
 	is_private = PG_ARGISNULL(2) ? false : PG_GETARG_BOOL(2);
 
-	WATCH_PRE(timeout, endtime, cycle);	
+	WATCH_PRE(timeout, endtime, cycle);
 	if (ora_lock_shmem(SHMEMMSGSZ, MAX_PIPES,MAX_EVENTS,MAX_LOCKS,false))
 	{
 		pipe *p;
@@ -1184,15 +1089,15 @@ dbms_pipe_create_pipe (PG_FUNCTION_ARGS)
 			if (!created)
 			{
 				LWLockRelease(shmem_lock);
-                    		ereport(ERROR,                                                                                                        
-                            		(errcode(ERRCODE_DUPLICATE_OBJECT),                                                                           
-                                	 errmsg("pipe creation error"),                                                                     
-                                	 errdetail("Pipe is registered.")));                       
+				ereport(ERROR,
+						(errcode(ERRCODE_DUPLICATE_OBJECT),
+						 errmsg("pipe creation error"),
+						 errdetail("Pipe is registered.")));
 			}
 			if (is_private)
 			{
 				char *user;
-				
+
 				p->uid = GetUserId();
 				user = (char*)DirectFunctionCall1(namein, CStringGetDatum(GetUserNameFromId(p->uid)));
 				p->creator = ora_sstrcpy(user);
@@ -1207,8 +1112,8 @@ dbms_pipe_create_pipe (PG_FUNCTION_ARGS)
 	}
 	WATCH_POST(timeout, endtime, cycle);
 	LOCK_ERROR();
-	
-	PG_RETURN_VOID();	
+
+	PG_RETURN_VOID();
 }
 
 
@@ -1224,26 +1129,23 @@ dbms_pipe_reset_buffer(PG_FUNCTION_ARGS)
 		pfree(output_buffer);
 		output_buffer = NULL;
 	}
-		
+
 	if (input_buffer != NULL)
 	{
 		pfree(input_buffer);
 		input_buffer = NULL;
 	}
 
-	reader = NULL;
-	writer = NULL;
-
 	PG_RETURN_VOID();
 }
 
 
 /*
- * Remove all stored messages in pipe. Remove implicit created 
+ * Remove all stored messages in pipe. Remove implicit created
  * pipe.
  */
 
-Datum 
+Datum
 dbms_pipe_purge (PG_FUNCTION_ARGS)
 {
 	text *pipe_name = PG_GETARG_TEXT_P(0);
@@ -1252,7 +1154,7 @@ dbms_pipe_purge (PG_FUNCTION_ARGS)
 	int cycle = 0;
 	int timeout = 10;
 
-	WATCH_PRE(timeout, endtime, cycle);	
+	WATCH_PRE(timeout, endtime, cycle);
 	if (ora_lock_shmem(SHMEMMSGSZ, MAX_PIPES,MAX_EVENTS,MAX_LOCKS,false))
 	{
 
@@ -1263,15 +1165,15 @@ dbms_pipe_purge (PG_FUNCTION_ARGS)
 	}
 	WATCH_POST(timeout, endtime, cycle);
 	LOCK_ERROR();
-	
+
 	PG_RETURN_VOID();
 }
 
 /*
  * Remove pipe if exists
- */ 
+ */
 
-Datum 
+Datum
 dbms_pipe_remove_pipe (PG_FUNCTION_ARGS)
 {
 	text *pipe_name = PG_GETARG_TEXT_P(0);
@@ -1280,7 +1182,7 @@ dbms_pipe_remove_pipe (PG_FUNCTION_ARGS)
 	int cycle = 0;
 	int timeout = 10;
 
-	WATCH_PRE(timeout, endtime, cycle);	
+	WATCH_PRE(timeout, endtime, cycle);
 	if (ora_lock_shmem(SHMEMMSGSZ, MAX_PIPES,MAX_EVENTS,MAX_LOCKS,false))
 	{
 
@@ -1291,7 +1193,7 @@ dbms_pipe_remove_pipe (PG_FUNCTION_ARGS)
 	}
 	WATCH_POST(timeout, endtime, cycle);
 	LOCK_ERROR();
-	
+
 	PG_RETURN_VOID();
 }
 
@@ -1307,9 +1209,9 @@ dbms_pipe_create_pipe_2 (PG_FUNCTION_ARGS)
 	Datum arg2 = PG_GETARG_DATUM(1);
 
 	DirectFunctionCall3(dbms_pipe_create_pipe,
-						    arg1,
-						    arg2,
-						    BoolGetDatum(false));
+						arg1,
+						arg2,
+						BoolGetDatum(false));
 
 	PG_RETURN_VOID();
 }
@@ -1320,28 +1222,28 @@ dbms_pipe_create_pipe_1 (PG_FUNCTION_ARGS)
 	Datum arg1 = PG_GETARG_DATUM(0);
 
 	DirectFunctionCall3(dbms_pipe_create_pipe,
-						    arg1,
-						    (Datum)0,
-						    BoolGetDatum(false));
+						arg1,
+						(Datum)0,
+						BoolGetDatum(false));
 
 	PG_RETURN_VOID();
 }
 
-Datum 
+Datum
 dbms_pipe_pack_message_integer(PG_FUNCTION_ARGS)
 {
 	/* Casting from int4 to numeric */
-	DirectFunctionCall1(dbms_pipe_pack_message_number, 
+	DirectFunctionCall1(dbms_pipe_pack_message_number,
 				DirectFunctionCall1(int4_numeric, PG_GETARG_DATUM(0)));
 
 	PG_RETURN_VOID();
 }
 
-Datum 
+Datum
 dbms_pipe_pack_message_bigint(PG_FUNCTION_ARGS)
 {
 	/* Casting from int8 to numeric */
-	DirectFunctionCall1(dbms_pipe_pack_message_number, 
+	DirectFunctionCall1(dbms_pipe_pack_message_number,
 				DirectFunctionCall1(int8_numeric, PG_GETARG_DATUM(0)));
 
 	PG_RETURN_VOID();
