@@ -1,5 +1,5 @@
 /*
-  This code implements one part of functonality of 
+  This code implements one part of functonality of
   free available library PL/Vision. Please look www.quest.com
 
   Original author: Steven Feuerstein, 1996 - 2002
@@ -58,7 +58,12 @@ PG_FUNCTION_INFO_V1(plvchr_char_name);
 PG_FUNCTION_INFO_V1(oracle_substr2);
 PG_FUNCTION_INFO_V1(oracle_substr3);
 
-char* char_names[] = {
+static text *ora_substr(Datum str, int start, int len);
+
+#define ora_substr_text(str, start, len) \
+	ora_substr(PointerGetDatum((str)), (start), (len))
+
+static const char* char_names[] = {
 	"NULL","SOH","STX","ETX","EOT","ENQ","ACK","DEL",
 	"BS",  "HT", "NL", "VT", "NP", "CR", "SO", "SI",
 	"DLE", "DC1","DC2","DC3","DC4","NAK","SYN","ETB",
@@ -66,17 +71,17 @@ char* char_names[] = {
 };
 
 #define NON_EMPTY_CHECK(str) \
-if (VARSIZE(str) - VARHDRSZ == 0) \
+if (VARSIZE_ANY_EXHDR(str) == 0) \
 	ereport(ERROR, \
-    		(errcode(ERRCODE_INVALID_PARAMETER_VALUE), \
-    		 errmsg("invalid parameter"), \
+			(errcode(ERRCODE_INVALID_PARAMETER_VALUE), \
+			 errmsg("invalid parameter"), \
 		 errdetail("Not allowed empty string.")));
 
 #define PARAMETER_ERROR(detail) \
 	ereport(ERROR, \
 		(errcode(ERRCODE_INVALID_PARAMETER_VALUE), \
 		 errmsg("invalid parameter"), \
-		 errdetail(detail))); 
+		 errdetail(detail)));
 
 
 #ifndef _pg_mblen
@@ -90,35 +95,9 @@ typedef enum
 	LAST
 }  position_mode;
 
-
+#if PG_VERSION_NUM < 80400
 text *
-ora_make_text(char *c)
-{
-	text *result;
-	int len;
-
-	len = strlen(c);
-	result = palloc(len + VARHDRSZ);
-	SET_VARSIZE(result, len + VARHDRSZ);
-	memcpy(VARDATA(result), c, len);
-
-	return result;
-}
-
-text*
-ora_clone_text(text *t)
-{
-	text *result;
-	
-	result = palloc(VARSIZE(t));
-	SET_VARSIZE(result, VARSIZE(t));
-	memcpy(VARDATA(result), VARDATA(t), VARSIZE(t) - VARHDRSZ);
-
-	return result;
-}
-
-text *
-ora_make_text_fix(char *c, int n)
+cstring_to_text_with_len(const char *c, int n)
 {
 	text *result;
 	result = palloc(n + VARHDRSZ);
@@ -127,10 +106,11 @@ ora_make_text_fix(char *c, int n)
 
 	return result;
 }
+#endif
 
 /*
  * Make substring, can handle negative start
- * 
+ *
  */
 
 
@@ -142,11 +122,11 @@ ora_mb_strlen(text *str, char **sizes, int **positions)
 	int sz;
 	char *p;
 	int cur = 0;
-	
-	p = (char*)VARDATA(str);
-	r_len = VARSIZE(str) - VARHDRSZ;
 
-    if (NULL != sizes)
+	p = VARDATA_ANY(str);
+	r_len = VARSIZE_ANY_EXHDR(str);
+
+	if (NULL != sizes)
 		*sizes = palloc(r_len * sizeof(char));
 	if (NULL != positions)
 		*positions = palloc(r_len * sizeof(int));
@@ -174,17 +154,17 @@ ora_mb_strlen1(text *str)
 	int c;
 	char *p;
 
-	r_len = VARSIZE(str) - VARHDRSZ;
-	
+	r_len = VARSIZE_ANY_EXHDR(str);
+
 	if (pg_database_encoding_max_length() == 1)
 		return r_len;
 
-	p = (char*)VARDATA(str);
+	p = VARDATA_ANY(str);
 	c = 0;
 	while (r_len > 0)
 	{
 		int sz;
-		
+
 		sz = _pg_mblen(p);
 		p += sz;
 		r_len -= sz;
@@ -194,198 +174,127 @@ ora_mb_strlen1(text *str)
 	return c;
 }
 
-text*
-ora_substr(text *str, int start, int len, bool valid_length)
+/*
+ * len < 0 means "length is not specified".
+ */
+static text *
+ora_substr(Datum str, int start, int len)
 {
-	text *result;
-	int l;
-	bool mb_encode;
-	char *sizes = NULL;
-	int *positions = NULL;
-
 	if (start == 0)
-		return ora_make_text("");
-
-	if (len < 0 && valid_length)
-		PARAMETER_ERROR("Third parameter is negative.");
-
-
-	mb_encode = pg_database_encoding_max_length() > 1;
-	if (!mb_encode)
-		l = VARSIZE(str) - VARHDRSZ;
-	else
-		l = ora_mb_strlen(str, &sizes, &positions);
-
-	start = start > 0 ? start : l + start + 1;
-	len = valid_length ? len : l - start + 1;
-	len = len + start - 1> l ? l - start + 1 : len;
-	len = len < 0 ? 0 : len;
-
-	if (!mb_encode)
+		start = 1;	/* 0 is interpreted as 1 */
+	else if (start < 0)
 	{
-		result = palloc(len + VARHDRSZ);
-		SET_VARSIZE(result, len + VARHDRSZ);
-		memcpy(VARDATA(result), ((char*)VARDATA(str))+start - 1, len);
-	}
-	else
-	{
-		int r_len;
-		int m_len;
-		int c_len;
-		int i;
-		int j;
-		char *p, *d;
-
-		r_len = VARSIZE(str) - VARHDRSZ;
-		m_len = len * pg_database_encoding_max_length();
-
-		if (r_len > m_len)
-			result = palloc(m_len + VARHDRSZ);
-		else
-			result = palloc(r_len + VARHDRSZ);
-		d =(char*) VARDATA(result);
-		
-		c_len = 0;
-		p = &((char*)VARDATA(str))[positions[start-1]];
-		for (i = start - 1; i < start+len-1; i++)
-		{
-			for (j = 0; j < sizes[i]; j++)
-				*d++ = *p++;
-			c_len += sizes[i]; 
-		}
-
-		SET_VARSIZE(result, c_len + VARHDRSZ);
+		text   *t;
+		int32	n;
 
+		t = DatumGetTextPP(str);
+		n = pg_mbstrlen_with_len(VARDATA_ANY(t), VARSIZE_ANY_EXHDR(t));
+		start = n + start + 1;
+		if (start <= 0)
+			return cstring_to_text("");
+		str = PointerGetDatum(t);	/* save detoasted text */
 	}
 
-	return result;
+	if (len < 0)
+		return DatumGetTextP(DirectFunctionCall2(text_substr_no_len,
+			str, Int32GetDatum(start)));
+	else
+		return DatumGetTextP(DirectFunctionCall3(text_substr,
+			str, Int32GetDatum(start), Int32GetDatum(len)));
 }
 
 /* simply search algorhitm - can be better */
 
-static int 
+static int
 ora_instr_mb(text *txt, text *pattern, int start, int nth)
 {
-	int c_len_txt, c_len_pat;
-	char *sizes_txt,  *sizes_pat;
-	int *pos_txt, *pos_pat;
-	char *str , *patt_f, *txt_p, *patt_p;
-	int dx, i, j;
-	int fzs_pat, fzs_txt;
-	int pos;
+	int			c_len_txt, c_len_pat;
+	int			b_len_pat;
+	int		   *pos_txt;
+	const char *str_txt, *str_pat;
+	int			beg, end, i, dx;
 
-	c_len_txt = ora_mb_strlen(txt, &sizes_txt, &pos_txt);
-	c_len_pat = ora_mb_strlen(pattern, &sizes_pat, &pos_pat);
-	fzs_txt = VARSIZE(txt) - VARHDRSZ;
-	fzs_pat = VARSIZE(pattern) - VARHDRSZ;
+	str_txt = VARDATA_ANY(txt);
+	c_len_txt = ora_mb_strlen(txt, NULL, &pos_txt);
+	str_pat = VARDATA_ANY(pattern);
+	b_len_pat = VARSIZE_ANY_EXHDR(pattern);
+	c_len_pat = pg_mbstrlen_with_len(str_pat, b_len_pat);
 
 	if (start > 0)
 	{
-		dx = 1; pos = 1;
-		str = &((char*)VARDATA(txt))[pos_txt[start-1]]; 
-		patt_f = (char*)VARDATA(pattern);
-		
+		dx = 1;
+		beg = start - 1;
+		end = c_len_txt - c_len_pat + 1;
+		if (beg >= end)
+			return 0;	/* out of range */
 	}
 	else
 	{
-		dx = -1; pos = c_len_txt+start+1;
-		str = &((char*)VARDATA(txt))[pos_txt[c_len_txt+ start]+sizes_txt[c_len_txt+ start]-1]; 
-		patt_f = ((char*)VARDATA(pattern)) + fzs_pat - 1;
+		dx = -1;
+		beg = Min(c_len_txt + start, c_len_txt - c_len_pat);
+		end = -1;
+		if (beg <= end)
+			return 0;	/* out of range */
 	}
 
-
-	for(i = 0; i < fzs_txt; i++)
+	for (i = beg; i != end; i += dx)
 	{
-		patt_p = patt_f;
-		txt_p = str;
-		for (j = 0; j < fzs_pat; j++)
-		{
-			if (*txt_p != *patt_p)
-				break;
-			txt_p += dx;
-			patt_p += dx;
-		}
-		if (j < fzs_pat)
-		{
-			str += dx*sizes_txt[pos-1];
-			pos += dx;
-		}
-		else
+		if (memcmp(str_txt + pos_txt[i], str_pat, b_len_pat) == 0)
 		{
 			if (--nth == 0)
-			{
-				return dx < 0 ? pos - c_len_pat + 1: pos + start-1;
-			}
-			else
-			{
-				str += (fzs_pat)*dx;
-				pos += dx*c_len_pat;
-			}
+				return i + 1;
 		}
-	}			
+	}
 
 	return 0;
 }
 
 
-int 
+int
 ora_instr(text *txt, text *pattern, int start, int nth)
 {
-	int i, j, len, len_p, dx;
-	char *str, *txt_p, *patt_p, *patt_f;
+	int			len_txt, len_pat;
+	const char *str_txt, *str_pat;
+	int			beg, end, i, dx;
 
 	if (nth <= 0)
 		PARAMETER_ERROR("Four parameter isn't positive.");
-	
-    /*
-     * Forward for multibyte strings
-     */
 
+	/* Forward for multibyte strings */
 	if (pg_database_encoding_max_length() > 1)
 		return ora_instr_mb(txt, pattern, start, nth);
 
-	len = VARSIZE(txt) - VARHDRSZ;
-	len_p = VARSIZE(pattern) - VARHDRSZ;	
-
+	str_txt = VARDATA_ANY(txt);
+	len_txt = VARSIZE_ANY_EXHDR(txt);
+	str_pat = VARDATA_ANY(pattern);
+	len_pat = VARSIZE_ANY_EXHDR(pattern);
 
 	if (start > 0)
 	{
 		dx = 1;
-		str = (char*)VARDATA(txt) + start - 1;
-		patt_f = (char*)VARDATA(pattern);
-		
+		beg = start - 1;
+		end = len_txt - len_pat + 1;
+		if (beg >= end)
+			return 0;	/* out of range */
 	}
 	else
 	{
 		dx = -1;
-		str = ((char*)VARDATA(txt)) + len + start;
-		patt_f = ((char*)VARDATA(pattern)) + len_p - 1;
+		beg = Min(len_txt + start, len_txt - len_pat);
+		end = -1;
+		if (beg <= end)
+			return 0;	/* out of range */
 	}
 
-	for(i = 0; i < len; i++)
+	for (i = beg; i != end; i += dx)
 	{
-		patt_p = patt_f;
-		txt_p = str;
-		for (j = 0; j < len_p; j++)
-		{
-			if (*txt_p != *patt_p)
-				break;
-			txt_p += dx;
-			patt_p += dx;
-		}
-		if (j < len_p)
-			str += dx;
-		else
+		if (memcmp(str_txt + i, str_pat, len_pat) == 0)
 		{
 			if (--nth == 0)
-			{
-				int off = str - VARDATA(txt) + 1;
-				return dx < 0 ? off - len_p + 1: off;
-			}
-			else
-				str += (len_p)*dx;
+				return i + 1;
 		}
-	}			
+	}
+
 	return 0;
 }
 
@@ -398,15 +307,15 @@ ora_instr(text *txt, text *pattern, int start, int nth)
  *  	RETURN VARCHAR;
  *
  * Purpouse:
- *   Normalize string - replace white chars by space, replace 
+ *   Normalize string - replace white chars by space, replace
  * spaces by space
  *
  ****************************************************************/
 
-Datum 
+Datum
 plvstr_normalize(PG_FUNCTION_ARGS)
 {
-	text *str = PG_GETARG_TEXT_P(0);
+	text *str = PG_GETARG_TEXT_PP(0);
 	text *result;
 	char *aux, *aux_cur;
 	int i, l;
@@ -415,14 +324,14 @@ plvstr_normalize(PG_FUNCTION_ARGS)
 	bool ignore_stsp = true;
 	bool mb_encode;
 	int sz;
-	
+
 	mb_encode = pg_database_encoding_max_length() > 1;
 
-	l = VARSIZE(str) - VARHDRSZ;
+	l = VARSIZE_ANY_EXHDR(str);
 	aux_cur = aux = palloc(l);
-	
+
 	write_spc = false;
-	cur = VARDATA(str);
+	cur = VARDATA_ANY(str);
 
 	for (i = 0; i < l; i++)
 	{
@@ -436,29 +345,29 @@ plvstr_normalize(PG_FUNCTION_ARGS)
 				break;
 			default:
 				/* ignore all other unvisible chars */
-				
+
 				if (mb_encode)
 				{
 					sz = _pg_mblen(cur);
 					if (sz > 1 || (sz == 1 && c > 32))
 					{
-					    int j;
-					    
-					    if (write_spc)
-					    {
-						    *aux_cur++ = ' ';
-						    write_spc = false;
+						int j;
 
-					    }
-					    for (j = 0; j < sz; j++)
-					    {
-						    *aux_cur++ = *cur++;
-					    }
-					    ignore_stsp = false;
-					    i += sz - 1;
+						if (write_spc)
+						{
+							*aux_cur++ = ' ';
+							write_spc = false;
+
+						}
+						for (j = 0; j < sz; j++)
+						{
+							*aux_cur++ = *cur++;
+						}
+						ignore_stsp = false;
+						i += sz - 1;
 					}
 					continue;
-				
+
 				}
 				else
 					if (c > 32)
@@ -472,7 +381,7 @@ plvstr_normalize(PG_FUNCTION_ARGS)
 						ignore_stsp = false;
 						continue;
 					}
-		
+
 		}
 		cur += 1;
 	}
@@ -481,7 +390,7 @@ plvstr_normalize(PG_FUNCTION_ARGS)
 	result = palloc(l+VARHDRSZ);
 	SET_VARSIZE(result, l + VARHDRSZ);
 	memcpy(VARDATA(result), aux, l);
-	
+
 	PG_RETURN_TEXT_P(result);
 }
 
@@ -495,27 +404,27 @@ plvstr_normalize(PG_FUNCTION_ARGS)
  *            start_in INTEGER)
  *   FUNCTION plvstr.instr (string_in VARCHAR, pattern VARCHAR,
  *            start_in INTEGER, nth INTEGER)
- *  	RETURN INT;
+ *            RETURN INT;
  *
  * Purpouse:
- *   Search pattern in string. 
+ *   Search pattern in string.
  *
  ****************************************************************/
 
 Datum
 plvstr_instr2 (PG_FUNCTION_ARGS)
 {
-	text *arg1 = PG_GETARG_TEXT_P(0);
-	text *arg2 = PG_GETARG_TEXT_P(1);
-	
+	text *arg1 = PG_GETARG_TEXT_PP(0);
+	text *arg2 = PG_GETARG_TEXT_PP(1);
+
 	PG_RETURN_INT32(ora_instr(arg1, arg2, 1, 1));
 }
 
 Datum
 plvstr_instr3 (PG_FUNCTION_ARGS)
 {
-	text *arg1 = PG_GETARG_TEXT_P(0);
-	text *arg2 = PG_GETARG_TEXT_P(1);
+	text *arg1 = PG_GETARG_TEXT_PP(0);
+	text *arg2 = PG_GETARG_TEXT_PP(1);
 	int arg3 = PG_GETARG_INT32(2);
 
 	PG_RETURN_INT32(ora_instr(arg1, arg2, arg3, 1));
@@ -524,8 +433,8 @@ plvstr_instr3 (PG_FUNCTION_ARGS)
 Datum
 plvstr_instr4 (PG_FUNCTION_ARGS)
 {
-	text *arg1 = PG_GETARG_TEXT_P(0);
-	text *arg2 = PG_GETARG_TEXT_P(1);
+	text *arg1 = PG_GETARG_TEXT_PP(0);
+	text *arg2 = PG_GETARG_TEXT_PP(1);
 	int arg3 = PG_GETARG_INT32(2);
 	int arg4 = PG_GETARG_INT32(3);
 
@@ -538,7 +447,7 @@ plvstr_instr4 (PG_FUNCTION_ARGS)
  *
  * Syntax:
  *   FUNCTION plvstr.is_prefix (string_in IN VARCHAR,
- *                    prefix_in VARCHAR, 
+ *                    prefix_in VARCHAR,
  *                    case_sensitive BOOL := true)
  *      RETURN bool;
  *   FUNCTION plvstr.is_prefix (num_in IN NUMERIC,
@@ -552,21 +461,21 @@ plvstr_instr4 (PG_FUNCTION_ARGS)
  ****************************************************************/
 
 
-Datum 
+Datum
 plvstr_is_prefix_text (PG_FUNCTION_ARGS)
 {
-	text *str = PG_GETARG_TEXT_P(0);
-	text *prefix = PG_GETARG_TEXT_P(1);
+	text *str = PG_GETARG_TEXT_PP(0);
+	text *prefix = PG_GETARG_TEXT_PP(1);
 	bool case_sens = PG_GETARG_BOOL(2);
 	bool mb_encode;
 
-	int str_len = VARSIZE(str) - VARHDRSZ;
-	int pref_len = VARSIZE(prefix) - VARHDRSZ;
+	int str_len = VARSIZE_ANY_EXHDR(str);
+	int pref_len = VARSIZE_ANY_EXHDR(prefix);
 
 	int i;
 	char *ap, *bp;
 
-	
+
 	mb_encode = pg_database_encoding_max_length() > 1;
 
 	if (mb_encode && !case_sens)
@@ -575,8 +484,8 @@ plvstr_is_prefix_text (PG_FUNCTION_ARGS)
 		prefix = (text*)DatumGetPointer(DirectFunctionCall1(lower, PointerGetDatum(prefix)));
 	}
 
-	ap = (char*)VARDATA(str);
-	bp = (char*)VARDATA(prefix);
+	ap = VARDATA_ANY(str);
+	bp = VARDATA_ANY(prefix);
 
 	for (i = 0; i < pref_len; i++)
 	{
@@ -612,9 +521,9 @@ plvstr_is_prefix_int (PG_FUNCTION_ARGS)
 			break;
 		}
 		n = n / 10;
-		
+
 	} while (n >= prefix);
-	
+
 	PG_RETURN_BOOL(result);
 }
 
@@ -633,9 +542,9 @@ plvstr_is_prefix_int64 (PG_FUNCTION_ARGS)
 			break;
 		}
 		n = n / 10;
-		
+
 	} while (n >= prefix);
-	
+
 	PG_RETURN_BOOL(result);
 }
 
@@ -645,8 +554,8 @@ plvstr_is_prefix_int64 (PG_FUNCTION_ARGS)
  *
  * Syntax:
  *   FUNCTION plvstr.rvrs (string_in IN VARCHAR,
- *					  start_in IN INTEGER := 1, 
- *					  end_in IN INTEGER := NULL) 
+ *					  start_in IN INTEGER := 1,
+ *					  end_in IN INTEGER := NULL)
  *  	RETURN VARCHAR2;
  *
  * Purpouse:
@@ -657,7 +566,7 @@ plvstr_is_prefix_int64 (PG_FUNCTION_ARGS)
 Datum
 plvstr_rvrs(PG_FUNCTION_ARGS)
 {
-	text *str = PG_GETARG_TEXT_P(0);
+	text *str = PG_GETARG_TEXT_PP(0);
 	int start = PG_GETARG_INT32(1);
 	int end = PG_GETARG_INT32(2);
 	int len, aux;
@@ -673,13 +582,13 @@ plvstr_rvrs(PG_FUNCTION_ARGS)
 		PG_RETURN_NULL();
 
 	mb_encode = pg_database_encoding_max_length() > 1;
-	
+
 	if (!mb_encode)
-		len = VARSIZE(str) - VARHDRSZ;
+		len = VARSIZE_ANY_EXHDR(str);
 	else
 		len = ora_mb_strlen(str, &sizes, &positions);
-	
-		
+
+
 
 	start = PG_ARGISNULL(1) ? 1 : start;
 	end = PG_ARGISNULL(2) ? (start < 0 ? -len : len) : end;
@@ -703,37 +612,37 @@ plvstr_rvrs(PG_FUNCTION_ARGS)
 		char *p;
 		int j;
 		int fz_size;
-		
-		fz_size = VARSIZE(str) - VARHDRSZ;
-		
+
+		fz_size = VARSIZE_ANY_EXHDR(str);
+
 		if ((max_size = (new_len*pg_database_encoding_max_length())) > fz_size)
-		    result = palloc(fz_size + VARHDRSZ);
+			result = palloc(fz_size + VARHDRSZ);
 		else
-		    result = palloc(max_size + VARHDRSZ);    
+			result = palloc(max_size + VARHDRSZ);
 		data = (char*) VARDATA(result);
-		
-		cur_size = 0; 
-		p = VARDATA(str);
+
+		cur_size = 0;
+		p = VARDATA_ANY(str);
 		for (i = end - 1; i>= start - 1; i--)
 		{
-		    for(j=0;j<sizes[i];j++)
-		        *data++ = *(p+positions[i]+j);
-		    cur_size += sizes[i];
+			for (j=0; j<sizes[i]; j++)
+				*data++ = *(p+positions[i]+j);
+			cur_size += sizes[i];
 		}
 		SET_VARSIZE(result, cur_size + VARHDRSZ);
-		
+
 	}
 	else
 	{
+		char *p = VARDATA_ANY(str);
 		result = palloc(new_len + VARHDRSZ);
-    		data = (char*) VARDATA(result);
+		data = (char*) VARDATA(result);
 		SET_VARSIZE(result, new_len + VARHDRSZ);
 
 		for (i = end - 1; i >= start - 1; i--)
-			*data++ = ((char*)VARDATA(str))[i];
-			
+			*data++ = p[i];
 	}
-	
+
 	PG_RETURN_TEXT_P(result);
 }
 
@@ -742,7 +651,7 @@ plvstr_rvrs(PG_FUNCTION_ARGS)
  * PLVstr.lpart
  *
  * Syntax:
- *   FUNCTION PLVstr.lpart (string_in IN VARCHAR, 
+ *   FUNCTION PLVstr.lpart (string_in IN VARCHAR,
  *					   divider_in IN VARCHAR,
  *					   start_in IN INTEGER := 1,
  *					   nth_in IN INTEGER := 1,
@@ -754,7 +663,7 @@ plvstr_rvrs(PG_FUNCTION_ARGS)
  *
  ****************************************************************/
 
-Datum 
+Datum
 plvstr_lpart (PG_FUNCTION_ARGS)
 {
 	text *str = PG_GETARG_TEXT_P(0);
@@ -768,12 +677,12 @@ plvstr_lpart (PG_FUNCTION_ARGS)
 	if (loc == 0)
 	{
 		if (all_if_notfound)
-			PG_RETURN_TEXT_P(ora_clone_text(str));
+			PG_RETURN_TEXT_P(TextPCopy(str));
 		else
 			PG_RETURN_NULL();
 	}
 	else
-		PG_RETURN_TEXT_P(ora_substr(str,1,loc-1,true));
+		PG_RETURN_TEXT_P(ora_substr_text(str, 1, loc-1));
 }
 
 
@@ -781,7 +690,7 @@ plvstr_lpart (PG_FUNCTION_ARGS)
  * PLVstr.rpart
  *
  * Syntax:
- *   FUNCTION PLVstr.rpart (string_in IN VARCHAR, 
+ *   FUNCTION PLVstr.rpart (string_in IN VARCHAR,
  *					   divider_in IN VARCHAR,
  *					   start_in IN INTEGER := 1,
  *					   nth_in IN INTEGER := 1,
@@ -793,7 +702,7 @@ plvstr_lpart (PG_FUNCTION_ARGS)
  *
  ****************************************************************/
 
-Datum 
+Datum
 plvstr_rpart (PG_FUNCTION_ARGS)
 {
 	text *str = PG_GETARG_TEXT_P(0);
@@ -807,12 +716,12 @@ plvstr_rpart (PG_FUNCTION_ARGS)
 	if (loc == 0)
 	{
 		if (all_if_notfound)
-			PG_RETURN_TEXT_P(ora_clone_text(str));
+			PG_RETURN_TEXT_P(TextPCopy(str));
 		else
 			PG_RETURN_NULL();
 	}
 	else
-		PG_RETURN_TEXT_P(ora_substr(str,loc+1,0,false));
+		PG_RETURN_TEXT_P(ora_substr_text(str, loc+1, -1));
 }
 
 
@@ -820,43 +729,43 @@ plvstr_rpart (PG_FUNCTION_ARGS)
  * PLVstr.lstrip
  *
  * Syntax:
- *   FUNCTION plvstr.lstrip (string_in IN VARCHAR, 
+ *   FUNCTION plvstr.lstrip (string_in IN VARCHAR,
  *							substring_in IN VARCHAR,
  *							num_in IN INTEGER := 1)
  *  	RETURN VARCHAR;
  *
  * Purpouse:
- *   Call this function to remove characters from the beginning 
+ *   Call this function to remove characters from the beginning
  * (left) of a string.
  *
  ****************************************************************/
 
-Datum 
+Datum
 plvstr_lstrip (PG_FUNCTION_ARGS)
 {
-	text *str = PG_GETARG_TEXT_P(0);
-	text *pat = PG_GETARG_TEXT_P(1);
+	text *str = PG_GETARG_TEXT_PP(0);
+	text *pat = PG_GETARG_TEXT_PP(1);
 	int num = PG_GETARG_INT32(2);
 	int count = 0;
 	int len_p, len_s, i;
 
 	char *str_p, *aux_str_p, *pat_p;
-	len_p = VARSIZE(pat) - VARHDRSZ;
-	len_s = VARSIZE(str) - VARHDRSZ;
+	len_p = VARSIZE_ANY_EXHDR(pat);
+	len_s = VARSIZE_ANY_EXHDR(str);
 
-	str_p = VARDATA(str);
+	str_p = VARDATA_ANY(str);
 	while (count < num)
 	{
-		pat_p = VARDATA(pat);
+		pat_p = VARDATA_ANY(pat);
 		aux_str_p = str_p;
-		
+
 		if (len_s < len_p)
 			break;
 
-		for(i = 0; i < len_p; i++)
+		for (i = 0; i < len_p; i++)
 			if (*aux_str_p++ != *pat_p++)
 				break;
-		
+
 		if (i >= len_p)
 		{
 			count++;
@@ -867,8 +776,8 @@ plvstr_lstrip (PG_FUNCTION_ARGS)
 		}
 		break;
 	}
-	
-	PG_RETURN_TEXT_P(ora_make_text_fix(str_p,len_s));
+
+	PG_RETURN_TEXT_P(cstring_to_text_with_len(str_p,len_s));
 }
 
 
@@ -876,44 +785,44 @@ plvstr_lstrip (PG_FUNCTION_ARGS)
  * PLVstr.rstrip
  *
  * Syntax:
- *   FUNCTION plvstr.rstrip (string_in IN VARCHAR, 
+ *   FUNCTION plvstr.rstrip (string_in IN VARCHAR,
  *							substring_in IN VARCHAR,
  *							num_in IN INTEGER := 1)
  *  	RETURN VARCHAR;
  *
  * Purpouse:
- *   Call this function to remove characters from the end 
+ *   Call this function to remove characters from the end
  * (right) of a string.
  *
  ****************************************************************/
 
-Datum 
+Datum
 plvstr_rstrip (PG_FUNCTION_ARGS)
 {
-	text *str = PG_GETARG_TEXT_P(0);
-	text *pat = PG_GETARG_TEXT_P(1);
+	text *str = PG_GETARG_TEXT_PP(0);
+	text *pat = PG_GETARG_TEXT_PP(1);
 	int num = PG_GETARG_INT32(2);
 	int count = 0;
 	int len_p, len_s, i;
 
 	char *str_p, *aux_str_p, *pat_p;
-	len_p = VARSIZE(pat) - VARHDRSZ;
-	len_s = VARSIZE(str) - VARHDRSZ;
+	len_p = VARSIZE_ANY_EXHDR(pat);
+	len_s = VARSIZE_ANY_EXHDR(str);
 
-	str_p = ((char*)VARDATA(str)) + len_s - 1;
+	str_p = VARDATA_ANY(str) + len_s - 1;
 
 	while (count < num)
 	{
-		pat_p = VARDATA(pat) + len_p - 1;
+		pat_p = VARDATA_ANY(pat) + len_p - 1;
 		aux_str_p = str_p;
-		
+
 		if (len_s < len_p)
 			break;
 
-		for(i = 0; i < len_p; i++)
+		for (i = 0; i < len_p; i++)
 			if (*aux_str_p-- != *pat_p--)
 				break;
-		
+
 		if (i >= len_p)
 		{
 			count++;
@@ -924,8 +833,8 @@ plvstr_rstrip (PG_FUNCTION_ARGS)
 		}
 		break;
 	}
-	
-	PG_RETURN_TEXT_P(ora_make_text_fix((char*)VARDATA(str),len_s));
+
+	PG_RETURN_TEXT_P(cstring_to_text_with_len(VARDATA_ANY(str),len_s));
 }
 
 
@@ -933,7 +842,7 @@ plvstr_rstrip (PG_FUNCTION_ARGS)
  * PLVstr.left
  *
  * Syntax:
- *   FUNCTION plvstr.left (string_in IN VARCHAR, 
+ *   FUNCTION plvstr.left (string_in IN VARCHAR,
  *							num_in INTEGER)
  *  	RETURN VARCHAR;
  *
@@ -944,7 +853,7 @@ plvstr_rstrip (PG_FUNCTION_ARGS)
  ****************************************************************/
 
 
-Datum 
+Datum
 plvstr_left (PG_FUNCTION_ARGS)
 {
 	text *str = PG_GETARG_TEXT_P(0);
@@ -952,8 +861,8 @@ plvstr_left (PG_FUNCTION_ARGS)
 	if (n < 0)
 		n = ora_mb_strlen1(str) + n;
 	n = n < 0 ? 0 : n;
-	
-	PG_RETURN_TEXT_P(ora_substr(str,1,n, true));
+
+	PG_RETURN_TEXT_P(ora_substr_text(str, 1, n));
 }
 
 
@@ -961,7 +870,7 @@ plvstr_left (PG_FUNCTION_ARGS)
  * PLVstr.right
  *
  * Syntax:
- *   FUNCTION plvstr.right (string_in IN VARCHAR, 
+ *   FUNCTION plvstr.right (string_in IN VARCHAR,
  *							num_in INTEGER)
  *  	RETURN VARCHAR;
  *
@@ -970,7 +879,7 @@ plvstr_left (PG_FUNCTION_ARGS)
  *
  ****************************************************************/
 
-Datum 
+Datum
 plvstr_right (PG_FUNCTION_ARGS)
 {
 	text *str = PG_GETARG_TEXT_P(0);
@@ -979,14 +888,14 @@ plvstr_right (PG_FUNCTION_ARGS)
 		n = ora_mb_strlen1(str) + n;
 	n = (n < 0) ? 0 : n;
 
-	PG_RETURN_TEXT_P(ora_substr(str,-n, 0, false));
+	PG_RETURN_TEXT_P(ora_substr_text(str, -n, -1));
 }
 
 /****************************************************************
  * PLVstr.substr2
  *
  * Syntax:
- *   FUNCTION plvstr.substr (string_in IN VARCHAR, 
+ *   FUNCTION plvstr.substr (string_in IN VARCHAR,
  *							start INTEGER)
  *  	RETURN VARCHAR;
  *
@@ -995,13 +904,10 @@ plvstr_right (PG_FUNCTION_ARGS)
  *
  ****************************************************************/
 
-Datum 
+Datum
 plvstr_substr2 (PG_FUNCTION_ARGS)
 {
-	text *str = PG_GETARG_TEXT_P(0);
-	int start = PG_GETARG_INT32(1);
-
-	PG_RETURN_TEXT_P(ora_substr(str,start, 0, false));
+	return oracle_substr2(fcinfo);
 }
 
 
@@ -1009,23 +915,19 @@ plvstr_substr2 (PG_FUNCTION_ARGS)
  * PLVstr.substr3
  *
  * Syntax:
- *   FUNCTION plvstr.substr (string_in IN VARCHAR, 
+ *   FUNCTION plvstr.substr (string_in IN VARCHAR,
  *							start INTEGER, len INTEGER)
  *  	RETURN VARCHAR;
  *
  * Purpouse:
- *   Returns len chars from start_in position 
+ *   Returns len chars from start_in position
  *
  ****************************************************************/
 
-Datum 
+Datum
 plvstr_substr3 (PG_FUNCTION_ARGS)
 {
-	text *str = PG_GETARG_TEXT_P(0);
-	int start = PG_GETARG_INT32(1);
-	int len = PG_GETARG_INT32(2);
-
-	PG_RETURN_TEXT_P(ora_substr(str,start, len, true));
+	return oracle_substr3(fcinfo);
 }
 
 
@@ -1033,20 +935,19 @@ plvstr_substr3 (PG_FUNCTION_ARGS)
  * PLVchr.nth
  *
  * Syntax:
- *   FUNCTION plvchr.nth (string_in IN VARCHAR, 
+ *   FUNCTION plvchr.nth (string_in IN VARCHAR,
  * 					 nth_in IN INTEGER)
  *  	RETURN VARCHAR;
  *
  * Purpouse:
- *   Call this function to return the Nth character in a string. 
+ *   Call this function to return the Nth character in a string.
  *
  ****************************************************************/
 
 Datum
 plvchr_nth (PG_FUNCTION_ARGS)
 {
-	PG_RETURN_TEXT_P(ora_substr(PG_GETARG_TEXT_P(0), 
-								PG_GETARG_INT32(1), 1, true));
+	PG_RETURN_TEXT_P(ora_substr(PG_GETARG_DATUM(0), PG_GETARG_INT32(1), 1));
 }
 
 
@@ -1054,7 +955,7 @@ plvchr_nth (PG_FUNCTION_ARGS)
  * PLVchr.first
  *
  * Syntax:
- *   FUNCTION plvchr.first (string_in IN VARCHAR, 
+ *   FUNCTION plvchr.first (string_in IN VARCHAR,
  *  	RETURN VARCHAR;
  *
  * Purpouse:
@@ -1065,7 +966,7 @@ plvchr_nth (PG_FUNCTION_ARGS)
 Datum
 plvchr_first (PG_FUNCTION_ARGS)
 {
-	PG_RETURN_TEXT_P(ora_substr(PG_GETARG_TEXT_P(0), 1, 1, true));
+	PG_RETURN_TEXT_P(ora_substr(PG_GETARG_DATUM(0), 1, 1));
 }
 
 
@@ -1073,7 +974,7 @@ plvchr_first (PG_FUNCTION_ARGS)
  * PLVchr.last
  *
  * Syntax:
- *   FUNCTION plvchr.last (string_in IN VARCHAR, 
+ *   FUNCTION plvchr.last (string_in IN VARCHAR,
  *  	RETURN VARCHAR;
  *
  * Purpouse:
@@ -1084,7 +985,7 @@ plvchr_first (PG_FUNCTION_ARGS)
 Datum
 plvchr_last (PG_FUNCTION_ARGS)
 {
-	PG_RETURN_TEXT_P(ora_substr(PG_GETARG_TEXT_P(0), -1, 1, true));
+	PG_RETURN_TEXT_P(ora_substr(PG_GETARG_DATUM(0), -1, 1));
 }
 
 
@@ -1092,9 +993,9 @@ plvchr_last (PG_FUNCTION_ARGS)
  * PLVchr.is_blank, plvchr.is_digit, ...
  *
  * Syntax:
- *   FUNCTION plvchr.is_kind (string_in IN VARCHAR, 
+ *   FUNCTION plvchr.is_kind (string_in IN VARCHAR,
  *      kind INT)
- *  	RETURN VARCHAR;
+ *          RETURN VARCHAR;
  *
  * Purpouse:
  *   Call this function to see if a character is blank, ...
@@ -1114,7 +1015,7 @@ is_kind(char c, int kind)
 		case 3:
 			return c == '\'';
 		case 4:
-			return 
+			return
 				(32 <= c && c <= 47) ||
 				(58 <= c && c <= 64) ||
 				(91 <= c && c <= 96) || (123 <= c && c <= 126);
@@ -1122,7 +1023,7 @@ is_kind(char c, int kind)
 			return ('A' <= c && c <= 'Z') || ('a' <= c && c <= 'z');
 		default:
 			PARAMETER_ERROR("Second parametr isn't in enum {1,2,3,4,5}");
-			return false; 
+			return false;
 	}
 }
 
@@ -1138,19 +1039,19 @@ plvchr_is_kind_i (PG_FUNCTION_ARGS)
 Datum
 plvchr_is_kind_a (PG_FUNCTION_ARGS)
 {
-	text *str = PG_GETARG_TEXT_P(0);
+	text *str = PG_GETARG_TEXT_PP(0);
 	int32 k = PG_GETARG_INT32(1);
 	char c;
 
-	NON_EMPTY_CHECK(str);	
+	NON_EMPTY_CHECK(str);
 	if (pg_database_encoding_max_length() > 1)
 	{
-		if (_pg_mblen(((char*)VARDATA(str))) > 1)
+		if (_pg_mblen(VARDATA_ANY(str)) > 1)
 			PG_RETURN_INT32( (k == 5) );
 	}
 
-	c = *((char*)VARDATA(str));
-	PG_RETURN_INT32(is_kind((char)c,k));
+	c = *VARDATA_ANY(str);
+	PG_RETURN_INT32(is_kind(c,k));
 }
 
 
@@ -1158,7 +1059,7 @@ plvchr_is_kind_a (PG_FUNCTION_ARGS)
  * PLVchr.char_name
  *
  * Syntax:
- *   FUNCTION plvchr.char_name (letter_in IN VARCHAR) 
+ *   FUNCTION plvchr.char_name (letter_in IN VARCHAR)
  *   	RETURN VARCHAR;
  *
  * Purpouse:
@@ -1169,19 +1070,18 @@ plvchr_is_kind_a (PG_FUNCTION_ARGS)
 Datum
 plvchr_char_name(PG_FUNCTION_ARGS)
 {
-	text *str = PG_GETARG_TEXT_P(0);
-	text *result; 
-	char c;
+	text *str = PG_GETARG_TEXT_PP(0);
+	text *result;
+	unsigned char c;
 
 	NON_EMPTY_CHECK(str);
-	c = *((char*)VARDATA(str));
+	c = (unsigned char)*(VARDATA_ANY(str));
 
-
-	if (c > 32 && _pg_mblen(((char*)VARDATA(str))) == 1)
-		result = ora_substr(str,1, 1, true);
+	if (c >= lengthof(char_names))
+		result = ora_substr_text(str, 1, 1);
 	else
-		result = ora_make_text(char_names[(int)c]);
-	
+		result = cstring_to_text(char_names[c]);
+
 	PG_RETURN_TEXT_P(result);
 }
 
@@ -1190,7 +1090,7 @@ plvchr_char_name(PG_FUNCTION_ARGS)
  * substr
  *
  * Syntax:
- *   FUNCTION substr (string, start_position, [length]) 
+ *   FUNCTION substr (string, start_position, [length])
  *   	RETURN VARCHAR;
  *
  * Purpouse:
@@ -1201,66 +1101,57 @@ plvchr_char_name(PG_FUNCTION_ARGS)
 Datum
 oracle_substr3(PG_FUNCTION_ARGS)
 {
-	text *str = PG_GETARG_TEXT_P(0);
-	int start = PG_GETARG_INT32(1);
-	int len = PG_GETARG_INT32(2);
-
+	int32	len = PG_GETARG_INT32(2);
 	if (len < 0)
 		PG_RETURN_NULL();
-
-	PG_RETURN_TEXT_P(ora_substr(str, start != 0 ? start : 1, len, true));
+	PG_RETURN_TEXT_P(ora_substr(PG_GETARG_DATUM(0), PG_GETARG_INT32(1), len));
 }
 
 Datum
 oracle_substr2(PG_FUNCTION_ARGS)
 {
-	text *str = PG_GETARG_TEXT_P(0);
-	int start = PG_GETARG_INT32(1);
-
-	PG_RETURN_TEXT_P(ora_substr(str, start != 0 ? start : 1, 0, false));
+	PG_RETURN_TEXT_P(ora_substr(PG_GETARG_DATUM(0), PG_GETARG_INT32(1), -1));
 }
 
 
-
-
 static text*
 ora_concat2(text *str1, text *str2)
 {
-	int l1;                                                                                                                                        
-	int l2;                                                                                                                                        
-	text *result;                                                                                                                                  
-                                                
-	l1 = VARSIZE(str1) - VARHDRSZ;                                                                                                                   
-	l2 = VARSIZE(str2) - VARHDRSZ;                                                                                                                   
-                                                                                                                                                   
-	result = palloc(l1+l2+VARHDRSZ);                                                                                                               
-	memcpy(VARDATA(result), VARDATA(str1), l1);                                                                                                      
-	memcpy(VARDATA(result) + l1, VARDATA(str2), l2);                                                                                                 
+	int l1;
+	int l2;
+	text *result;
+
+	l1 = VARSIZE_ANY_EXHDR(str1);
+	l2 = VARSIZE_ANY_EXHDR(str2);
+
+	result = palloc(l1+l2+VARHDRSZ);
+	memcpy(VARDATA(result), VARDATA_ANY(str1), l1);
+	memcpy(VARDATA(result) + l1, VARDATA_ANY(str2), l2);
 	SET_VARSIZE(result, l1 + l2 + VARHDRSZ);
-                            
-	return result;                                                                                                                       
+
+	return result;
 }
 
 
 static text*
 ora_concat3(text *str1, text *str2, text *str3)
 {
-	int l1;                                                                                                                                        
-	int l2;                                                                                                                                        
+	int l1;
+	int l2;
 	int l3;
-	text *result;                                                                                                                                  
-                                                
-	l1 = VARSIZE(str1) - VARHDRSZ;
-	l2 = VARSIZE(str2) - VARHDRSZ;
-	l3 = VARSIZE(str3) - VARHDRSZ;
-                                                                                                                                                   
-	result = palloc(l1+l2+l3+VARHDRSZ);                                                                                                               
-	memcpy(VARDATA(result), VARDATA(str1), l1);
-	memcpy(VARDATA(result) + l1, VARDATA(str2), l2);
-	memcpy(VARDATA(result) + l1+l2, VARDATA(str3), l3);
+	text *result;
+
+	l1 = VARSIZE_ANY_EXHDR(str1);
+	l2 = VARSIZE_ANY_EXHDR(str2);
+	l3 = VARSIZE_ANY_EXHDR(str3);
+
+	result = palloc(l1+l2+l3+VARHDRSZ);
+	memcpy(VARDATA(result), VARDATA_ANY(str1), l1);
+	memcpy(VARDATA(result) + l1, VARDATA_ANY(str2), l2);
+	memcpy(VARDATA(result) + l1+l2, VARDATA_ANY(str3), l3);
 	SET_VARSIZE(result, l1 + l2 + l3 + VARHDRSZ);
-                            
-	return result;                                                                                                                       
+
+	return result;
 }
 
 
@@ -1280,7 +1171,7 @@ ora_concat3(text *str1, text *str2, text *str3)
  *
  ****************************************************************/
 
-Datum 
+Datum
 plvstr_swap(PG_FUNCTION_ARGS)
 {
 	text *string_in;
@@ -1290,21 +1181,21 @@ plvstr_swap(PG_FUNCTION_ARGS)
 	int v_len;
 
 	if (PG_ARGISNULL(0))
-	        PG_RETURN_NULL();
+		PG_RETURN_NULL();
 	else
-	        string_in = PG_GETARG_TEXT_P(0);
+		string_in = PG_GETARG_TEXT_P(0);
 
 	if (PG_ARGISNULL(1))
-	        PG_RETURN_NULL();
+		PG_RETURN_NULL();
 	else
-	        replace_in = PG_GETARG_TEXT_P(1);
+		replace_in = PG_GETARG_TEXT_P(1);
 
 	if (!PG_ARGISNULL(2))
-	        start_in = PG_GETARG_INT32(2);
+		start_in = PG_GETARG_INT32(2);
 
 	if (PG_ARGISNULL(3))
-	        oldlen_in = ora_mb_strlen1(replace_in);
-        else
+		oldlen_in = ora_mb_strlen1(replace_in);
+	else
 		oldlen_in = PG_GETARG_INT32(3);
 
 	v_len =  ora_mb_strlen1(string_in);
@@ -1312,15 +1203,15 @@ plvstr_swap(PG_FUNCTION_ARGS)
 	start_in = start_in > 0 ? start_in : v_len + start_in + 1;
 
 	if (start_in == 0 || start_in > v_len)
-	        PG_RETURN_TEXT_P(ora_clone_text(string_in));
+		PG_RETURN_TEXT_P(TextPCopy(string_in));
 	else if (start_in == 1)
-	        PG_RETURN_TEXT_P(ora_concat2(replace_in, 
-					      ora_substr(string_in, oldlen_in+1,0, false)));
+		PG_RETURN_TEXT_P(ora_concat2(
+			replace_in, ora_substr_text(string_in, oldlen_in+1, -1)));
 	else
-	        PG_RETURN_TEXT_P(ora_concat3(ora_substr(string_in, 1, start_in-1, true),
-					       replace_in,
-					       ora_substr(string_in, start_in + oldlen_in, 0, false)));
-
+		PG_RETURN_TEXT_P(ora_concat3(
+			ora_substr_text(string_in, 1, start_in - 1),
+			replace_in,
+			ora_substr_text(string_in, start_in + oldlen_in, -1)));
 }
 
 /****************************************************************
@@ -1329,14 +1220,14 @@ plvstr_swap(PG_FUNCTION_ARGS)
  * Find the Substring Between Start and End Locations
  *
  * Syntax:
- *     FUNCTION plvstr.betwn (string_in IN VARCHAR2, 
- *       start_in IN INTEGER, 
+ *     FUNCTION plvstr.betwn (string_in IN VARCHAR2,
+ *       start_in IN INTEGER,
  *       end_in IN INTEGER,
  *       inclusive IN BOOLEAN := TRUE)
  *      RETURN VARCHAR2;
  *
- *     FUNCTION plvstr.betwn (string_in IN VARCHAR2, 
- *       start_in IN VARCHAR2, 
+ *     FUNCTION plvstr.betwn (string_in IN VARCHAR2,
+ *       start_in IN VARCHAR2,
  *       end_in IN VARCHAR2 := NULL,
  *       startnth_in IN INTEGER := 1,
  *       endnth_in IN INTEGER := 1,
@@ -1345,14 +1236,14 @@ plvstr_swap(PG_FUNCTION_ARGS)
  *      RETURN VARCHAR2;
  *
  * Purpouse:
- *   Call this function to extract a sub-string from a string. This 
- * function is overloaded. You can either provide the start and end 
+ *   Call this function to extract a sub-string from a string. This
+ * function is overloaded. You can either provide the start and end
  * locations or you can provide start and end substrings.
  *
  ****************************************************************/
 
 
-Datum 
+Datum
 plvstr_betwn_i(PG_FUNCTION_ARGS)
 {
 	text *string_in = PG_GETARG_TEXT_P(0);
@@ -1360,9 +1251,9 @@ plvstr_betwn_i(PG_FUNCTION_ARGS)
 	int end_in = PG_GETARG_INT32(2);
 	bool inclusive = PG_GETARG_BOOL(3);
 
-	if ((start_in < 0 && end_in > 0) ||   
-	    (start_in > 0 && end_in < 0) ||
-	    (start_in > end_in))
+	if ((start_in < 0 && end_in > 0) ||
+		(start_in > 0 && end_in < 0) ||
+		(start_in > end_in))
 		PARAMETER_ERROR("Wrong positions.");
 
 	if (start_in < 0)
@@ -1378,16 +1269,16 @@ plvstr_betwn_i(PG_FUNCTION_ARGS)
 		end_in -= 1;
 
 		if (start_in > end_in)
-			PG_RETURN_TEXT_P(ora_make_text(""));
-	}    
+			PG_RETURN_TEXT_P(cstring_to_text(""));
+	}
 
-	PG_RETURN_TEXT_P(ora_substr(string_in, 
-				    start_in, 
-				    end_in - start_in + 1, true));
+	PG_RETURN_TEXT_P(ora_substr_text(string_in,
+									 start_in,
+									 end_in - start_in + 1));
 }
 
 
-Datum 
+Datum
 plvstr_betwn_c(PG_FUNCTION_ARGS)
 {
 	text *string_in;
@@ -1402,10 +1293,10 @@ plvstr_betwn_c(PG_FUNCTION_ARGS)
 	int v_end;
 
 	if (PG_ARGISNULL(0) || PG_ARGISNULL(1) ||
-	    PG_ARGISNULL(3) || PG_ARGISNULL(4) ||
-	    PG_ARGISNULL(5) || PG_ARGISNULL(6))
+		PG_ARGISNULL(3) || PG_ARGISNULL(4) ||
+		PG_ARGISNULL(5) || PG_ARGISNULL(6))
 		PG_RETURN_NULL();
-	
+
 
 	string_in = PG_GETARG_TEXT_P(0);
 	start_in = PG_GETARG_TEXT_P(1);
@@ -1414,12 +1305,12 @@ plvstr_betwn_c(PG_FUNCTION_ARGS)
 	endnth_in = PG_GETARG_INT32(4);
 	inclusive = PG_GETARG_BOOL(5);
 	gotoend = PG_GETARG_BOOL(6);
-	
+
 	if (startnth_in == 0)
 	{
 		v_start = 1;
 		v_end = ora_instr(string_in, end_in, 1, endnth_in);
-	}	
+	}
 	else
 	{
 		v_start = ora_instr(string_in, start_in, 1, startnth_in);
@@ -1438,15 +1329,15 @@ plvstr_betwn_c(PG_FUNCTION_ARGS)
 	}
 	else
 		v_end += (ora_mb_strlen1(end_in) - 1);
-	
-	if (((v_start > v_end) && (v_end > 0)) || 
-	    (v_end <= 0 && !gotoend))
+
+	if (((v_start > v_end) && (v_end > 0)) ||
+		(v_end <= 0 && !gotoend))
 		PG_RETURN_NULL();
 
 	if (v_end <= 0)
 		v_end = ora_mb_strlen1(string_in);
 
-	PG_RETURN_TEXT_P(ora_substr(string_in, 
-				    v_start, 
-				    v_end - v_start + 1, true));
+	PG_RETURN_TEXT_P(ora_substr_text(string_in,
+									 v_start,
+									 v_end - v_start + 1));
 }
